using System.Collections.Generic;
using Jypeli;
using System.Linq;

namespace Merirosvo_peli;

/// @author joseh
/// @version 30.03.2025
public class Merirosvo_peli : PhysicsGame
{
    private const double PALLON_NOPEUS = 250;
    private const double PELAAJAN_NOPEUS = 400;
    private const double PALLON_KOKO = 50;
    private const int PISTEEN_ARVO = 1;
    private const double KANUUNANKUULAN_TODENNARKYISYYS = 0.5;
    private const double PALLON_LISAYS_VALI = 1;
    private const double PELAAJAN_KOKO = 80;
    private const double PELAAJAN_Y_SIJAINTI = 60;
    private const double PISTELASKURIN_Y_SIJAINTI = -50;

    private PhysicsObject pelaaja;
    private IntMeter pisteet;
    private Timer pallojenLisaaja;
    private List<PhysicsObject> kaikkiPallot = new List<PhysicsObject>();
    
    private static readonly Image PelaajaKuva = LoadImage("Merirosvo.png");
    private static readonly Image EsineKuva = LoadImage("Kolikko");
    private static readonly Image EsteKuva = LoadImage("Kanuunankuula");

    /// <summary>
    /// Aloittaa pelin kutsumalla kentän luontia, ohjaimia ja pallojen lisäystä.
    /// </summary>
    public override void Begin()
    {
        LuoKentta();
        AsetaOhjaimet();
        LisaaPistelaskuri();
        AloitaPallojenLuonti();
    }

    /// <summary>
    /// Luo pelikentän taustan, reunat ja pelaajan.
    /// </summary>
    private void LuoKentta()
    {
        Level.Background.Color = Color.SkyBlue;
        Level.CreateBorders(1.0, true);

        pelaaja = new PhysicsObject(PELAAJAN_KOKO, PELAAJAN_KOKO)
        {
            Shape = Shape.Rectangle,
            X = 0,
            Y = Level.Bottom + PELAAJAN_Y_SIJAINTI,
            Restitution = 0.0,
            Tag = "pelaaja",
        };
        pelaaja.Image = PelaajaKuva;
        
            
        Add(pelaaja);

        PhysicsObject pohja = Level.CreateBottomBorder();
        pohja.Tag = "pohja";
    }


    /// <summary>
    /// Asettaa näppäimistöohjaimet pelaajan liikuttamiseen.
    /// </summary>
    private void AsetaOhjaimet()
    {
        Keyboard.Listen(Key.Left, ButtonState.Down, LiikutaPelaajaa, "Liikuta vasemmalle", -PELAAJAN_NOPEUS);
        Keyboard.Listen(Key.Right, ButtonState.Down, LiikutaPelaajaa, "Liikuta oikealle", PELAAJAN_NOPEUS);
        Keyboard.Listen(Key.Left, ButtonState.Released, PysaytaLiike, "Pysäytä liike");
        Keyboard.Listen(Key.Right, ButtonState.Released, PysaytaLiike, "Pysäytä liike");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, Exit, "Lopeta peli");
    }


    /// <summary>
    /// Liikuttaa pelaajaa annetulla nopeudella.
    /// </summary>
    private void LiikutaPelaajaa(double nopeus)
    {
        pelaaja.Velocity = new Vector(nopeus, 0);
    }


    /// <summary>
    /// Pysäyttää pelaajan liikkeen.
    /// </summary>
    private void PysaytaLiike()
    {
        pelaaja.Velocity = Vector.Zero;
        if (pelaaja.Left <= Level.Left) pelaaja.X = Level.Left + pelaaja.Width / 2;
        if (pelaaja.Right >= Level.Right) pelaaja.X = Level.Right - pelaaja.Width / 2;
    }

    /// <summary>
    /// Lisää pistelaskurin peliin.
    /// </summary>
    private void LisaaPistelaskuri()
    {
        pisteet = new IntMeter(0);

        Label pistelaskuri = new Label
        {
            TextColor = Color.Black,
            X = -Level.Width / 2 + 100,
            Y = Level.Top + PISTELASKURIN_Y_SIJAINTI
        };
        pistelaskuri.BindTo(pisteet);
        Add(pistelaskuri);
    }


    /// <summary>
    /// Aloittaa pallojen luomisen ajastimella.
    /// </summary>
    private void AloitaPallojenLuonti()
    {
        pallojenLisaaja = new Timer
        {
            Interval = PALLON_LISAYS_VALI
        };
        pallojenLisaaja.Timeout += LuoPallojaAalloittain;
        pallojenLisaaja.Start();
    }

    /// <summary>
    /// Luo viisi palloa kerralla peliin.
    /// </summary>
    private void LuoPallojaAalloittain()
    {
        for (int i = 0; i < 5; i++)
        {
            LisaaPallo();
        }
    }


    /// <summary>
    /// Lisää yhden pallon peliin satunnaiseen sijaintiin.
    /// </summary>
    private void LisaaPallo()
    {
        double x = RandomGen.NextDouble(Level.Left + 50, Level.Right - 50);
        PhysicsObject pallo = new PhysicsObject(PALLON_KOKO, PALLON_KOKO)
        {
            Position = new Vector(x, Level.Top),
            Velocity = new Vector(0, -PALLON_NOPEUS),
            Restitution = 0.0,
            IgnoresCollisionResponse = true
        };

        if (RandomGen.NextDouble(0.0, 1.0) > KANUUNANKUULAN_TODENNARKYISYYS)
        {
            pallo.Tag = "pistepallo";
            pallo.Image = EsineKuva;
        }
        else
        {
            pallo.Tag = "kanuunankuula";
            pallo.Image = EsteKuva;
        }

        kaikkiPallot.Add(pallo);
        Add(pallo);
        AddCollisionHandler(pallo, "pelaaja", PelaajaOsuiPalloon);
        AddCollisionHandler(pallo, "pohja", PalloOsuiPohjaan);
    }


    /// <summary>
    /// Käsittelee pelaajan ja pallon törmäyksen.
    /// </summary>
    private void PelaajaOsuiPalloon(PhysicsObject pallo, PhysicsObject pelaaja)
    {
        if (pallo.Tag == "kanuunankuula")
        {
            ResetoiPeli();
        }
        else if (pallo.Tag == "pistepallo")
        {
            pisteet.Value += PISTEEN_ARVO;
            pallo.Destroy();
            kaikkiPallot.Remove(pallo);
        }
    }


    /// <summary>
    /// Käsittelee pallon osumisen pohjaan.
    /// </summary>
    private void PalloOsuiPohjaan(PhysicsObject pallo, PhysicsObject pohja)
    {
        pallo.Destroy();
        kaikkiPallot.Remove(pallo);
    }


    /// <summary>
    /// Nollaa pelin tilan.
    /// </summary>
    private void ResetoiPeli()
    {
        pisteet.Value = 0;

        foreach (var pallo in kaikkiPallot.ToList())
        {
            pallo.Destroy();
        }

        kaikkiPallot.Clear();

        pelaaja.Position = new Vector(0, Level.Bottom + PELAAJAN_Y_SIJAINTI);
        pelaaja.Velocity = Vector.Zero;
    }
}
